import importlib
import logging
from pip._internal.operations import freeze

import fastapi

from .build_info import BUILD_INFO
from .config import AppConfig, Config
from . import schemas


log = logging.getLogger(__name__)


def _import(module_name: str, def_name: str):
    module = importlib.import_module(module_name)
    return getattr(module, def_name)


class EngineGateway(fastapi.FastAPI):

    def _mount_from_app(self, config: AppConfig):
        if config.app is None:
            return
        try:
            app = _import(config.module, config.app)
            self.mount(config.path, app)
        except (ImportError, AttributeError) as e:
            log.error('Error importing "%s" from "%s": %s',
                      config.app, config.module, e)

    def _mount_from_factory(self, config: AppConfig):
        if config.factory is None:
            return
        try:
            factory = _import(config.module, config.factory)
            app = factory(**(config.kwargs or {}))
            self.mount(config.path, app)
        except (ImportError, AttributeError) as e:
            log.error('Error importing "%s" from "%s": %s',
                      config.app, config.module, e)

    def mount_gateway_app(self, config: AppConfig):
        if config.factory is not None:
            self._mount_from_factory(config)
        else:
            self._mount_from_app(config)


def create_info(config: Config) -> schemas.GatewayInfo:
    packages = list(freeze.freeze())
    return schemas.GatewayInfo(
        version=BUILD_INFO.version,
        built_at=BUILD_INFO.built_at,
        mounts=config.mounts,
        packages=packages,
    )



# Generated file
import collections

BuildInfo = collections.namedtuple(
    'BuildInfo',
    ['version', 'built_at', 'sha', 'branch', 'tag'],
)

BUILD_INFO = BuildInfo(
    version='v0.1.0~59ebd0c',
    built_at='2025-11-14 14:06:04Z',
    sha='59ebd0c1bd9b89db44b8f0131adf124618ffe42e',
    branch='HEAD',
    tag='v0.1.0',
)



import logging
import os
import pathlib

import pydantic
import yaml

from .consts import DEFAULT_ENCODING


CONFIG_FILE_NAME = os.environ.get('CONFIG_FILE')
CONFIG_FILE = pathlib.Path.cwd() / 'config.yaml'
if CONFIG_FILE_NAME:
    CONFIG_FILE = pathlib.Path(CONFIG_FILE_NAME)

log = logging.getLogger(__name__)


def _is_true(value: str) -> bool:
    return value.lower() in ('1', 'true', 'yes', 'on')


class AppConfig(pydantic.BaseModel):
    path: str
    module: str
    app: str | None
    factory: str | None
    kwargs: dict | None


class SentryConfig(pydantic.BaseModel):
    dsn: str | None
    traces_sample_rate: float | None
    profiles_sample_rate: float | None
    max_breadcrumbs: int | None
    environment: str | None
    tags: dict[str, str]
    aws_lambda: bool = False


class GatewayConfig(pydantic.BaseModel):
    title: str
    root_path: str
    info_enabled: bool
    info_token: str | None


class Config:

    def __init__(self):
        self.gateway: GatewayConfig = GatewayConfig(
            title='Engine Gateway',
            root_path='',
            info_enabled=False,
            info_token=None,
        )
        self.sentry: SentryConfig = SentryConfig(
            dsn=None,
            traces_sample_rate=None,
            profiles_sample_rate=None,
            max_breadcrumbs=None,
            environment=None,
            aws_lambda=False,
            tags={},
        )
        self.mounts: list[AppConfig] = []

    def load_yaml(self):
        if not CONFIG_FILE.exists():
            log.warning('Config file %s does not exist; skipping YAML loading.',
                        CONFIG_FILE)
            return
        with open(CONFIG_FILE, 'r', encoding=DEFAULT_ENCODING) as f:
            data = yaml.safe_load(f)

        mounts_data = data.get('mounts', {})
        for path, details in mounts_data.items():
            app_config = AppConfig(
                path=path,
                module=details.get('module'),
                app=details.get('app'),
                factory=details.get('factory'),
                kwargs=details.get('kwargs', {})
            )
            self.mounts.append(app_config)
        self._load_yaml_gateway(data)
        self._load_yaml_sentry(data)

    def _load_yaml_gateway(self, data: dict):
        gateway_data = data.get('gateway', {})
        if 'title' in gateway_data:
            self.gateway.title = gateway_data['title']
        if 'root_path' in gateway_data:
            self.gateway.root_path = gateway_data['root_path']
        info_data = gateway_data.get('info', None)
        if info_data is not None:
            self.gateway.info_enabled = info_data.get('enabled', True)
            self.gateway.info_token = info_data.get('token', None)

    def _load_yaml_sentry(self, data: dict):
        sentry_data = data.get('sentry', None)
        if sentry_data is not None:
            if 'dsn' in sentry_data:
                self.sentry.dsn = sentry_data['dsn']
            if 'traces_sample_rate' in sentry_data:
                self.sentry.traces_sample_rate = float(sentry_data['traces_sample_rate'])
            if 'profiles_sample_rate' in sentry_data:
                self.sentry.profiles_sample_rate = float(sentry_data['profiles_sample_rate'])
            if 'max_breadcrumbs' in sentry_data:
                self.sentry.max_breadcrumbs = int(sentry_data['max_breadcrumbs'])
            if 'environment' in sentry_data:
                self.sentry.environment = sentry_data['environment']
            if 'tags' in sentry_data:
                self.sentry.tags = sentry_data['tags']
            if 'aws_lambda' in sentry_data:
                self.sentry.aws_lambda = bool(sentry_data['aws_lambda'])

    def load_env(self):
        if 'GATEWAY_TITLE' in os.environ:
            self.gateway.title = os.getenv('GATEWAY_TITLE', 'Engine Gateway')
        if 'GATEWAY_ROOT_PATH' in os.environ:
            self.gateway.root_path = os.getenv('GATEWAY_ROOT_PATH', '')
        if 'GATEWAY_INFO_ENABLED' in os.environ:
            self.gateway.info_enabled = _is_true(os.getenv('GATEWAY_INFO_ENABLED', 'false'))
        if 'GATEWAY_INFO_TOKEN' in os.environ:
            self.gateway.info_token = os.getenv('GATEWAY_INFO_TOKEN')

        if 'SENTRY_DSN' in os.environ:
            self.sentry.dsn = os.getenv('SENTRY_DSN')
        if 'SENTRY_TRACES_SAMPLE_RATE' in os.environ:
            self.sentry.traces_sample_rate = float(os.getenv('SENTRY_TRACES_SAMPLE_RATE', '1.0'))
        if 'SENTRY_PROFILES_SAMPLE_RATE' in os.environ:
            self.sentry.profiles_sample_rate = float(os.getenv('SENTRY_PROFILES_SAMPLE_RATE', '1.0'))
        if 'SENTRY_MAX_BREADCRUMBS' in os.environ:
            self.sentry.max_breadcrumbs = int(os.getenv('SENTRY_MAX_BREADCRUMBS', '100'))
        if 'SENTRY_ENVIRONMENT' in os.environ:
            self.sentry.environment = os.getenv('SENTRY_ENVIRONMENT')
        if 'SENTRY_AWS_LAMBDA' in os.environ:
            self.sentry.aws_lambda = _is_true(os.getenv('SENTRY_AWS_LAMBDA', 'false'))



import pydantic

from .config import AppConfig


class GatewayInfo(pydantic.BaseModel):
    version: str
    built_at: str | None
    mounts: list[AppConfig] = pydantic.Field(default_factory=list)
    packages: list[str] = pydantic.Field(default_factory=list)



import fastapi

from .build_info import BuildInfo
from .config import Config
from .logic import EngineGateway, create_info
from .sentry import init_sentry


def create_app(config: Config | None = None) -> EngineGateway:
    if config is None:
        config = Config()
        config.load_yaml()
        config.load_env()

    init_sentry(config)

    app = EngineGateway(
        title=config.gateway.title,
        version=BuildInfo.version,
        root_path=config.gateway.root_path.rstrip('/'),
    )

    if config.gateway.info_enabled:
        def get_info(request: fastapi.Request):
            token = config.gateway.info_token
            if token:
                auth_header = request.headers.get('Authorization')
                if auth_header != f'Bearer {token}':
                    raise fastapi.HTTPException(status_code=401, detail='Unauthorized')
            return create_info(config)

        app.add_api_route(
            '/info',
            endpoint=get_info,
            methods=['GET'],
        )

    for app_config in config.mounts:
        app.mount_gateway_app(app_config)

    return app



import logging

import sentry_sdk

from .build_info import BuildInfo
from .config import Config

log = logging.getLogger(__name__)


def init_sentry(config: Config) -> None:
    if not config.sentry.dsn:
        log.info('Sentry DSN not provided; skipping Sentry initialization.')
        return

    try:
        integrations = []
        if config.sentry.aws_lambda:
            # pylint: disable-next=import-outside-toplevel
            from sentry_sdk.integrations.aws_lambda import AwsLambdaIntegration
            integrations.append(AwsLambdaIntegration())
        sentry_sdk.init(
            dsn=config.sentry.dsn,
            traces_sample_rate=float(config.sentry.traces_sample_rate or 1.0),
            profiles_sample_rate=float(config.sentry.profiles_sample_rate or 1.0),
            max_breadcrumbs=int(config.sentry.max_breadcrumbs or sentry_sdk.consts.DEFAULT_MAX_BREADCRUMBS),
            release=BuildInfo.version,
            environment=config.sentry.environment,
            integrations=integrations,
        )
        for key, value in config.sentry.tags.items():
            sentry_sdk.set_tag(key, value)
        log.info('Sentry initialized successfully.')
    except Exception as e:
        log.error('Failed to initialize Sentry: %s',
                  e, exc_info=e)
